package packageForLT05;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Ellipse2D;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;

/**
 * <p>
 * The class encapsulates a view of a SimpleThreeDotModel (where a "view" is a
 * visual representation of data). This view is based on a widget container and
 * can be instantiated and displayed.
 * </p>
 * 
 * <p>
 * The view is a single container widget that contains one widget, the
 * DotDrawingPanel. This widget provides a single bounded two-dimensional plane.
 * The dots in the data model are each drawn on the plane in their specified
 * location and with their specified size. The user space is mapped directly to
 * the device space.
 * </p>
 * 
 * <p>
 * The only constructor this class has is one that requires a
 * SimpleThreeDotModel object. It doesn't make sense to create this view without
 * its data model.
 * </p>
 * 
 * <p>
 * This view is capable of listening to data models and will install itself on
 * the passed data model. Any time the data model changes, this view will
 * repaint itself.
 * </p>
 * 
 * @author mb
 * 
 */
public class DotView extends JFrame implements DotModelListener {
	private DotModel theModel;
	private int width;
	private int height;

	private JPanel thePanel;

	private DotView() {
	}

	/**
	 * Constructs a view as specified in the class description. The view gets
	 * installed upon the passed model.
	 * 
	 * @param model
	 */
	public DotView(DotModel model) {
		int width = model.getMaxX();
		int height = model.getMaxY();
		this.setTitle("My Frame");
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setSize(width, height);
		this.setMinimumSize(new Dimension(width, height));
		this.setBackground(Color.WHITE);

		thePanel = new DotDrawingWidget(model);
		this.add(thePanel);

		this.pack();
		this.setVisible(true);

		theModel = model;
		// does this view need to update itself in response to any changes
		// in the model? If so, install a listener here
		model.addListener(this);

	}

	/**
	 * This method renders the graphics of this view.
	 * 
	 * The method is invoked any time the view is repainted. This method will
	 * cause all of the widgets contain within it to be rendered (according to
	 * that widget's paintComponent(Graphics) method.
	 */
	public void paintComponent(Graphics g) {
		// super.paintComponent(g); // yes, this should be the first statement,
		// but the code works as-is without this potentially confusing part

		// the paintComponent method for each of the child widgets of this
		// widget will be invoked
	}

	public Dimension getPreferredSize() {
		return new Dimension(width, height);
	}

	/**
	 * <p>
	 * Method that changes this view in response to any change in the data
	 * model. If the data model changes, the view repaints itself anew.
	 * <p/>
	 * 
	 * <p>
	 * This method is invoked to signal to this listener that the model (upon
	 * which this listener has been installed) has been changed. This provides
	 * an opportunity for the listener to something in response to any change to
	 * the model.
	 * </p>
	 * 
	 * <p>
	 * <b>Who invokes this method?</b> Think of who has been given access to
	 * this view... The documentation for the constructor specifies that this
	 * view has been installed upon the specified data model. So the data model
	 * is able to invoke this method within its inner workings.
	 * </p>
	 */
	public void changed() {
		// what should happen if we come to learn that the model has changed? we
		// should repaint ourself.
		this.repaint();
	}

}
